--Author Manaleaf - Sargeras
--Core Calcs for Stat Weights

function LFLT_A.Calc()
	--Returns the UnitID of a raid member with the given name (works off server)
	function getUnit(name)
		--Manual Override for Replay System
		if LFLT_T.manualOverride.getUnit then
			return LFLT_T.manualOverride.getUnit
		end
		local destUnit
		local grpCount = GetNumGroupMembers()
		if UnitInRaid("player") then
			for i = 1, grpCount  do
				local name2,realm2 = UnitName("raid"..i)
				if realm2 then name2 = name2 .. "-" .. realm2 end
				
				if name2 == name then
					destUnit = "raid" .. i
					break
				end
			end   
		elseif UnitInParty("player") then
			for i = 1, grpCount  do
				local name2,realm2 = UnitName("party"..i)
				if realm2 then name2 = name2 .. "-" .. realm2 end
				
				if name2 == name then
					destUnit = "party" .. i
					break
				end
			end 
		elseif UnitName("player") == name then
			destUnit = "player" 
		end
		return destUnit
	end
	
	--Returns the current number of Player casted hots on the unit
	function LFLT.hotCounter(destUnit)
		--Manual Override for Replay System
		if LFLT_T.manualOverride.hotCounter then
			return LFLT_T.manualOverride.hotCounter
		end
		local hCount = 0 
		if not UnitExists(destUnit) then
			return -1 --Failure Flag
		end
		for k,v in ipairs(LFLT_T.hotList) do
			if UnitBuff(destUnit, GetSpellInfo(v), nil, "PLAYER") then hCount = hCount + 1 end 
		end
		return hCount
	end

	--Clears the current healing and stat values	
	function LFLT.clearStats()
		LFLT_T.score.cur_int_hpc = 0
		LFLT_T.score.cur_int_hpt = 0
		LFLT_T.score.cur_mst_hpc = 0
		LFLT_T.score.cur_mst_hpt = 0
		LFLT_T.score.cur_hst_hpc = 0
		LFLT_T.score.cur_hst_hpt = 0
		LFLT_T.score.cur_crt_hpc = 0
		LFLT_T.score.cur_crt_hpt = 0
		LFLT_T.score.cur_vrs_hpc = 0
		LFLT_T.score.cur_vrs_hpt = 0
		LFLT_T.score.cur_lch_hpc = 0
		LFLT_T.score.cur_lch_hpt = 0
	end
	
	--Clears the Current and Total healing and stat values	
	function LFLT.clearAllStats()
		LFLT_T.score.cur_int_hpc = 0
		LFLT_T.score.cur_int_hpt = 0
		LFLT_T.score.cur_mst_hpc = 0
		LFLT_T.score.cur_mst_hpt = 0
		LFLT_T.score.cur_hst_hpc = 0
		LFLT_T.score.cur_hst_hpt = 0
		LFLT_T.score.cur_crt_hpc = 0
		LFLT_T.score.cur_crt_hpt = 0
		LFLT_T.score.cur_vrs_hpc = 0
		LFLT_T.score.cur_vrs_hpt = 0
		LFLT_T.score.cur_lch_hpc = 0
		LFLT_T.score.cur_lch_hpt = 0
		
		LFLT_T.score.ttl_int_hpc = 0
		LFLT_T.score.ttl_int_hpt = 0
		LFLT_T.score.ttl_mst_hpc = 0
		LFLT_T.score.ttl_mst_hpt = 0
		LFLT_T.score.ttl_hst_hpc = 0
		LFLT_T.score.ttl_hst_hpt = 0
		LFLT_T.score.ttl_crt_hpc = 0
		LFLT_T.score.ttl_crt_hpt = 0
		LFLT_T.score.ttl_vrs_hpc = 0
		LFLT_T.score.ttl_vrs_hpt = 0
		LFLT_T.score.ttl_lch_hpc = 0
		LFLT_T.score.ttl_lch_hpt = 0
	end
	
	--Retrieves and Sets values related to Artifact Traits
	function LFLT_T.updateArtifact()
		local hide 
		local seedsPoints = 0
		if IsEquippedItem(128306) == true then
			if ArtifactFrame then
				if not ArtifactFrame:IsShown() then 
					SocketInventoryItem(16)
					hide = true
				end
				--131 is Trait: Seeds if the World Tree according to C_ArtifactUI.GetPowers()
				seedsPoints, _ = C_ArtifactUI.GetPowerInfo(131).currentRank
				if hide then HideUIPanel(ArtifactFrame) end
			else --If game has not loaded Artifact Frame yet.
				C_Timer.After(0.25, function() LFLT_T.updateArtifact() end)
				return
			end
		end
		if not seedsPoints then 
			C_Timer.After(0.25, function() LFLT_T.updateArtifact() end)
		else
			LFLT_T.regrowthBaseCrt = LFLT_T.INITREGROWTHBASECRIT + .08 * seedsPoints
		end
	end
	
	----------------------------------------DO NOT LET RECORDS IN TRANSITION SET LEECH MODE
	function LFLT_T.transitionRecords(stats)
		--only one iteration of this function/break recursive calls if there are no records in transition
		if LFLT_T.activeTransition == true or #LFLT_T.recordsInTransition == 0 then return end
		local ttab = {} --transition table
		for i,record in ipairs(LFLT_T.recordsInTransition) do
			ttab[i] = {}
			for k,v in ipairs(record) do
				ttab[i].k =  v
			end
			tremove(LFLT_T.recordsInTransition, i)
		end
		--Allow other iterations of this function after reading and clearing table.
		LFLT_T.activeTransition = false
		for k,v in ipairs(ttab) do
			--21 possible arguments of a "COMBAT_LOG_EVENT_UNFILTERED" event.
			LFLT_T.statCalc(stats, 	v[1], v[2], v[3], v[4], v[5], v[6], v[7], v[8], v[9], v[10], v[11], 
									v[12], v[13], v[14], v[15], v[16], v[17], v[18], v[19], v[20], v[21])
		end
		LFLT_T.transitionRecords(stats)
	end
		
	--Stores Spell info into the active record 
	function LFLT_T.recordInsert(heal, sName, sType, hCount)
		--Manual Override for Replay System
		if LFLT_T.manualOverride.recordInsert then return end
		local log = LFLT.sessionRecords.active.spells	
		if log[sName] then
			if log[sName].heal[hCount] then
				log[sName].heal[hCount] = log[sName].heal[hCount] + heal
			else 
				log[sName].heal[hCount] = heal
			end
		else 
			log[sName] = {}
			log[sName]["heal"] = {}
			log[sName]["heal"][hCount] = heal
			log[sName]["sType"] = sType
		end
	end  
	
	--[[Updates Character Stats
	Table LFLT.sessionRecords stores a list of the player's stats
	Table spellRecords is a Subtable which stores stats about healing 
		done by spells while player has a specific set of stats
	While function is invoked statCalc entries cannot be added to a table.
		They will be stored in a temp table and passed into the new active LFLT.sessionRecords
	]]  
	function LFLT_T.updateStats()
		LFLT.sessionRecords.active = nil
		local hstRatingPerc = 1 + (GetCombatRating(20) / LFLT_T.HSTRATINGCONV) --Haste percent from rating
		local hstPercentage = 1 + UnitSpellHaste("player") / 100
		local curStats = {
		int = GetSpellBonusDamage(4),
		mst = 1 + GetMasteryEffect() / 100,
		hst = hstPercentage,
		crt = 1 + GetCritChance() / 100,
		vrs = 1 + (GetCombatRatingBonus(CR_VERSATILITY_DAMAGE_DONE) + GetVersatilityBonus(CR_VERSATILITY_DAMAGE_DONE)) / 100,
		lch =  1 + GetCombatRatingBonus(17),
		hstMod = hstPercentage / hstRatingPerc, --Multiplicative modifier for haste
		}
		
		--Crit Effect Multiplier
		LFLT_T.critBonusOutput = LFLT_T.baseCritBonusOutput
		if IsEquippedItem("Drape of Shame") then 
			curStats.critBonusOutput = LFLT_T.critBonusOutput * 1.1
		else
			curStats.critBonusOutput = LFLT_T.critBonusOutput
		end
		
		--find matching record
		for _,record  in ipairs(LFLT.sessionRecords) do
			if curStats[int] == record[int]
			and curStats[mst] == record[mst]
			and curStats[hst] == record[hst]
			and curStats[crt] == record[crt]
			and curStats[vrs] == record[vrs]
			and curStats[lch] == record[lch]
			and curStats[hstMod] == record[hstMod] then
				LFLT.sessionRecords.active = record
				break
			end
		end
		
		--Could not find matching record, creating new one
		if not LFLT.sessionRecords.active then
			local tablePos = #LFLT.sessionRecords + 1
			LFLT.sessionRecords[tablePos] = {
				int = curStats.int,
				mst = curStats.mst,
				hst = curStats.hst,
				crt = curStats.crt,
				vrs = curStats.vrs,
				lch = curStats.lch,
				hstMod = curStats.hstMod,
				critBonusOutput = curStats.critBonusOutput,
				spells = {},
			}
			LFLT.sessionRecords.active = LFLT.sessionRecords[tablePos]
		end
		LFLT_T.transitionRecords(curStats)
	end
		
	--Core Calcs for Leaflet
	--@param stats: Overrides player stats in case of delayed calculation (due to stat change)
	function LFLT_T.statCalc(stats, ...)
		--[[print(	"Spell Type:", select(2, ...),
				"\nPGUID:", select(4, ...),
				"\nTGUID:", select(9, ...),
				"\nSpell Name:", select(13, ...),
				"\nHeal:", select(15, ...),
				"\nOverheal:", select(16, ...),
				"\nCrtFlag:", select(18, ...))
				
		1 nil, 
		2 healData.sType, 		--spell name
		3 nil, 
		4 UnitGUID("player"),  --Player GUID
		5 nil, 
	    6 nil, 
		7 nil, 
		8 nil, 
		9 nil, 					--Target UnitID (left nil)
		10 nil, 
		11 nil, 
		12 nil,
		13 spellName,			--spell name
		14 nil,
		15 math.max(adjDirHeal, adjHotHeal), 		 --heal amount 
		16 0, 					--Overheal
		17 nil,
		18 false)				--Crit Flag (always left false
		]]		
		
		--Return if casting unit is not player[4] or if overheal is detected[16]
		if select(4, ...) ~= UnitGUID("player") or select(16, ... ) ~= 0 then return end
		
		--If using current player stats and current player stats are unavilable due to stat change
		if not stats then
			if not LFLT.sessionRecords.active then
				local record = ""
				local newRecordPos = #LFLT_T.recordsInTransition +1
				LFLT_T.recordsInTransition[newRecordPos] = {}
				for i=1,select("#", ...) do
						LFLT_T.recordsInTransition[newRecordPos][i] = select(i, ...)
				end
				return
			else
			--if using current player stats
				stats = {}
				stats.int = LFLT.sessionRecords.active.int
				stats.hst = LFLT.sessionRecords.active.hst 
				stats.mst = LFLT.sessionRecords.active.mst
				stats.crt = LFLT.sessionRecords.active.crt
				stats.vrs = LFLT.sessionRecords.active.vrs
				stats.lch = LFLT.sessionRecords.active.lch
				stats.hstMod = LFLT.sessionRecords.active.hstMod
				stats.critBonusOutput = LFLT.sessionRecords.active.critBonusOutput
			end
		end
		local sType 		= select(2, ...)			--spell type
		local destUnit 		= getUnit(select(9, ...))	--Target UnitID
		local sName 		= select(13, ...)			--spell name
		
		local hstFlag, sklFlag, lchFlag
		--Hot Spells (haste effected)
		if sType == "SPELL_PERIODIC_HEAL" then	
			--LFLT_T.leechHaste[destUnit] = true
			if sName == LFLT_T.spells.rejuvenation
			or sName == LFLT_T.spells.germination
			or sName == LFLT_T.spells.lifebloom
			or sName == LFLT_T.spells.wildgrowth
			or sName == LFLT_T.spells.springblossoms
			or sName == LFLT_T.spells.cultivation
			or sName == LFLT_T.spells.cenarionward
			or sName == LFLT_T.spells.regrowth .. "Hot"
				then hstFlag = true 
			elseif sName == LFLT_T.spells.regrowth
				then hstFlag = true
				sName = LFLT_T.spells.regrowth .. "Hot"
			end
			
			
			--Direct Healing Spells (Mostly not Haste Effected)   
		elseif sType == "SPELL_HEAL" then 
			if sName == LFLT_T.spells.leech then
				--hstFlag = LFLT_T.leechHaste[destUnit]
				--LFLT_T.leechHaste[destUnit] = nil
				--lchFlag = true
				
			elseif sName == LFLT_T.spells.efflorescence then 
				hstFlag = true
				--LFLT_T.leechHaste[destUnit] = true
				
			elseif sName == LFLT_T.spells.regrowth then 
				sklFlag = 1
				hstFlag = false
				--LFLT_T.leechHaste[destUnit] = true
				
			elseif sName == LFLT_T.spells.livingseed then
				sklFlag = 2
				hstFlag = false
				--LFLT_T.leechHaste[destUnit] = false
				
			elseif sName == LFLT_T.spells.swiftmend
			or sName == LFLT_T.spells.healingtouch
			or sName == LFLT_T.spells.lifebloom
			or sName == LFLT_T.spells.dreamwalker
			then 
				hstFlag = false 
				--LFLT_T.leechHaste[destUnit] = true
			
			elseif sName == LFLT_T.spells.tranquility 
			and LFLT.option.ignoreTranquilityEnable then
				hstFlag = false 
			end
		end
	
		--If hstFlag == nil, 
		--healing was not done by a spell in the above listing. 
		--ie: Ysera's gift is uneffected by secondaries		
		if hstFlag ~= nil then 
			local heal 	= select(15, ...)	--heal amount (including overheal)
			local crtFlag = select(18, ...)	--Crit Flag
			--Mastery Percentage
			local hCount = LFLT.hotCounter(destUnit)
			if hCount == -1 then return end
			local mst = 1 + ((stats.mst - 1) * hCount)
			
			--Get Base Heal
			if crtFlag == true then
				heal = heal / (2 * stats.critBonusOutput)
			end
			--Crit Percentage (Bonus)
			if sklFlag == 1 then 
				if overHeal ~= 0 then return end
				stats.crt = (stats.crt +  LFLT_T.regrowthBaseCrt) * stats.critBonusOutput
			else  
				stats.crt = stats.crt * stats.critBonusOutput
			end
			
			--Spell Coeff.
			local HPCT, intHPC, mstHPC, hstHPC, crtHPC, vrsHPC, intHPT, mstHPT, hstHPT, crtHPT, vrsHPT, lchHPC, lchHPT
			--Heal Per Cast Time 
			
			----Haste Calc
			if hstFlag then
				hstHPC = (heal / stats.hst) * (stats.hstMod / LFLT_T.HSTRATINGCONV)
				hstHPT = (heal / (stats.hst^2)) * stats.hstMod * (LFLT_T.HSTRATINGCONV + stats.hst) / LFLT_T.HSTRATINGCONVSPCVAL
			else
				hstHPC = 0
				hstHPT = (heal / stats.hst) * (stats.hstMod / LFLT_T.HSTRATINGCONV)
			end
			
			----HPC: Mastery Calc
			mstHPC = heal / mst * (hCount / LFLT_T.MSTRATINGCONV )	
			----HPC: Crit Calc
			crtHPC = heal / stats.crt * (stats.critBonusOutput / LFLT_T.CRTRATINGCONV)
			----HPC: Versatility Calc
			vrsHPC = heal / (stats.vrs * LFLT_T.VRSRATINGCONV)
			----HPC: Intellect Calc
			intHPC = heal / stats.int * LFLT_T.ARMORTYPEBONUS 
						
			----HPCT: Mastery Calc
			mstHPT = mstHPC * stats.hst
			----HPCT: Crit Calc
			crtHPT = crtHPC * stats.hst
			----HPCT: Versatility Calc
			vrsHPT = vrsHPC * stats.hst
			----HPCT: Intellect Calc
			intHPT = intHPC * stats.hst
			
			----Leech Calc
			if lchFlag then
				lchHPC = heal / (stats.lch * LFLT_T.LCHRATINGCONV)
				lchHPT = lchHPC * stats.hst
				LFLT_T.score.cur_lch_hpc = LFLT_T.score.cur_lch_hpc + lchHPC
				LFLT_T.score.cur_lch_hpt = LFLT_T.score.cur_lch_hpt + lchHPT
			end
			--allocate
			--HPC:Current Encounter Healing Score Allocation
			LFLT_T.score.cur_int_hpc = LFLT_T.score.cur_int_hpc + intHPC
			LFLT_T.score.cur_mst_hpc = LFLT_T.score.cur_mst_hpc + mstHPC
			LFLT_T.score.cur_hst_hpc = LFLT_T.score.cur_hst_hpc + hstHPC
			LFLT_T.score.cur_crt_hpc = LFLT_T.score.cur_crt_hpc + crtHPC
			LFLT_T.score.cur_vrs_hpc = LFLT_T.score.cur_vrs_hpc + vrsHPC

			--HPC:Total Healing Score Allocation
			LFLT_T.score.ttl_int_hpc = LFLT_T.score.ttl_int_hpc + intHPC
			LFLT_T.score.ttl_mst_hpc = LFLT_T.score.ttl_mst_hpc + mstHPC
			LFLT_T.score.ttl_hst_hpc = LFLT_T.score.ttl_hst_hpc + hstHPC
			LFLT_T.score.ttl_crt_hpc = LFLT_T.score.ttl_crt_hpc + crtHPC
			LFLT_T.score.ttl_vrs_hpc = LFLT_T.score.ttl_vrs_hpc + vrsHPC

			--HPCT:Current Encounter Healing Score Allocation
			LFLT_T.score.cur_int_hpt = LFLT_T.score.cur_int_hpt + intHPT
			LFLT_T.score.cur_mst_hpt = LFLT_T.score.cur_mst_hpt + mstHPT
			LFLT_T.score.cur_hst_hpt = LFLT_T.score.cur_hst_hpt + hstHPT
			LFLT_T.score.cur_crt_hpt = LFLT_T.score.cur_crt_hpt + crtHPT
			LFLT_T.score.cur_vrs_hpt = LFLT_T.score.cur_vrs_hpt + vrsHPT

			--HPCT:Total Healing Score Allocation
			LFLT_T.score.ttl_int_hpt = LFLT_T.score.ttl_int_hpt + intHPT
			LFLT_T.score.ttl_mst_hpt = LFLT_T.score.ttl_mst_hpt + mstHPT
			LFLT_T.score.ttl_hst_hpt = LFLT_T.score.ttl_hst_hpt + hstHPT
			LFLT_T.score.ttl_crt_hpt = LFLT_T.score.ttl_crt_hpt + crtHPT
			LFLT_T.score.ttl_vrs_hpt = LFLT_T.score.ttl_vrs_hpt + vrsHPT
			
			LFLT_T.updateDisplay(1)
			LFLT_T.recordInsert(heal, sName, sType, hCount)
		end  
	end
end
